@@ -5,8 +5,9 @@ from collections import deque, defaultdict
 from heapq import heappop, heappush
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard
-from trytond.pyson import Eval, Not, Equal
+from trytond.pyson import Eval
 from trytond.transaction import Transaction
+from trytond.pool import Pool
 
 
 def intfloor(x):
@@ -17,22 +18,24 @@ class Work(ModelSQL, ModelView):
     _name = 'project.work'
 
     predecessors = fields.Many2Many('project.predecessor_successor',
-            'successor', 'predecessor', 'Predecessors',
-            domain=[
-                ('parent', '=', Eval('parent')),
-                ('id', '!=', Eval('active_id')),
-            ])
+        'successor', 'predecessor', 'Predecessors',
+        domain=[
+            ('parent', '=', Eval('parent')),
+            ('id', '!=', Eval('id')),
+            ],
+        depends=['parent', 'id'])
     successors = fields.Many2Many('project.predecessor_successor',
-            'predecessor', 'successor', 'Successors',
-            domain=[
-                ('parent', '=', Eval('parent')),
-                ('id', '!=', Eval('active_id')),
-            ])
+        'predecessor', 'successor', 'Successors',
+        domain=[
+            ('parent', '=', Eval('parent')),
+            ('id', '!=', Eval('id')),
+            ],
+        depends=['parent', 'id'])
     leveling_delay = fields.Float("Leveling Delay")
     back_leveling_delay = fields.Float("Leveling Delay")
     allocations = fields.One2Many('project.allocation', 'work', 'Allocations',
-            states={
-                'invisible': Not(Equal(Eval('type'), 'task')),
+        states={
+            'invisible': Eval('type') != 'task',
             }, depends=['type'])
     duration = fields.Function(fields.Float('Duration'), 'get_function_fields')
     early_start_time = fields.DateTime("Early Start Time", readonly=True)
@@ -86,7 +89,7 @@ class Work(ModelSQL, ModelView):
         :return: a dictionary with all field names as key and
             a dictionary as value with id as key
         '''
-        req_ref_obj = self.pool.get('res.request.reference')
+        req_ref_obj = Pool().get('res.request.reference')
 
         cursor = Transaction().cursor
 
@@ -171,8 +174,8 @@ class Work(ModelSQL, ModelView):
         return res
 
     def set_function_fields(self, ids, name, value):
-        request_obj = self.pool.get('res.request')
-        req_ref_obj = self.pool.get('res.request.reference')
+        request_obj = Pool().get('res.request')
+        req_ref_obj = Pool().get('res.request.reference')
 
         if name == 'requests':
             works = self.browse(ids)
@@ -186,20 +189,34 @@ class Work(ModelSQL, ModelView):
                 operator = v[0]
 
                 target_ids = len(v) > 1 and v[1] or []
-                if operator == 'set':
+                if operator == 'create':
+                    to_link.append(request_obj.create(v[1]))
+                elif operator == 'write':
+                    request_obj.write(v[1], v[2])
+                elif operator == 'delete':
+                    request_obj.delete(v[1])
+                elif operator == 'delete_all':
+                    target_ids = []
+                    for record_id in ids:
+                        ref_ids = req_ref_obj.search([
+                                ('reference', '=',
+                                    'project.work,%s' % record_id),
+                                ])
+                        refs = req_ref_obj.browse(ref_ids)
+                        target_ids.extend(ref.request.id for ref in refs)
+                    request_obj.delete(target_ids)
+                elif operator == 'unlink':
+                    to_unlink.extend((i for i in target_ids if i in currents))
+                elif operator == 'add':
+                    to_link.extend((i for i in target_ids if i not in
+                        currents))
+                elif operator == 'unlink_all':
+                    to_unlink.extend(currents)
+                elif operator == 'set':
                     to_link.extend((i for i in target_ids
                         if i not in currents))
                     to_unlink.extend((i for i in currents
                         if i not in target_ids))
-                elif operator == 'add':
-                    to_link.extend((i for i in target_ids if i not in
-                        currents))
-                elif operator == 'unlink':
-                    to_unlink.extend((i for i in target_ids if i in currents))
-                elif operator == 'unlink_all':
-                    to_unlink.extend(currents)
-                elif operator == 'delete':
-                    request_obj.delete(to_delete)
                 else:
                     raise Exception('Operation not supported')
 
@@ -656,7 +673,7 @@ class PredecessorSuccessor(ModelSQL):
             ondelete='CASCADE', required=True, select=1)
 
     def write(self, ids, values):
-        work_obj = self.pool.get('project.work')
+        work_obj = Pool().get('project.work')
         res = super(PredecessorSuccessor, self).write(ids, values)
 
         for work_id in values.itervalues():
@@ -666,7 +683,7 @@ class PredecessorSuccessor(ModelSQL):
         return res
 
     def delete(self, ids):
-        work_obj = self.pool.get('project.work')
+        work_obj = Pool().get('project.work')
         if isinstance(ids, (int, long)):
             ids = [ids]
 
@@ -691,7 +708,7 @@ class PredecessorSuccessor(ModelSQL):
         return res
 
     def create(self, values):
-        work_obj = self.pool.get('project.work')
+        work_obj = Pool().get('project.work')
         ps_id = super(PredecessorSuccessor, self).create(values)
 
         pred_succ = self.browse(ps_id)
@@ -720,7 +737,7 @@ class Leveling(Wizard):
         }
 
     def _leveling(self, data):
-        work_obj = self.pool.get('project.work')
+        work_obj = Pool().get('project.work')
         work_obj.create_leveling(data['id'])
         return {}
 
